#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys_defs.h>
#include <mymalloc.h>
#include "auth.h"
#include "optm.h"

static void usage() {
  fprintf(stderr, "Usage:\n\tauthcleanup timeout_in_minutes [dict_name]\n\nExample:\n\tauth_cleanup 10 hash:/etc/postfix/test\n");
  exit(1);
}

int main (int argc, char ** argv) {
  DBTYPE * db;
  register int i;
  char * scheme=NULL, * dbname;
  char * key, * value;
#ifdef USE_OPTIMIZERS
  optm_t * optm;
  optmdb_t * optmdb;
#endif
  int timeout=0, now;
  int status;
  int tmp;

  /* Parse command line parameters */
  for(i=1; i < argc; i++) {
    switch(i) {
      case 1:
	timeout=atoi(argv[i])*60;
	break;

      case 2: 
	scheme=argv[i];
	break;
      
      default:
	usage();
	break;
    }
  }

  /* Verify presence of all needed parameters */
  if(scheme == NULL) {
    scheme=getenv("LOGINDB");
    if(scheme == NULL)
      scheme = DEFAULTDB;
  }

  scheme = mystrdup(scheme);
  if(scheme == NULL || timeout == 0)
    usage();

  split_up(scheme, &scheme, &dbname);
  now=time(NULL);

#ifdef USE_OPTIMIZERS
  /* Look up if we have an optimized procedure for the indicated scheme */
  optm=optm_lookup(scheme);
  if(optm != NULL && optm->remove != NULL) {
    optmdb=optm->open(dbname);
    status=optm->remove(optmdb, now-timeout);
    optm->close(optmdb);

    return status;
  }
#endif 

  /* Use the standard postfix interface */
  db=db_open3(scheme, dbname);
  status=db_seq(db, DICT_SEQ_FUN_FIRST, (void *)&key, (void *)&value);
  while (!status && key != NULL && value != NULL) {
    tmp=atoi(value);
    if(tmp && tmp <= (now-timeout))
      /* how happy are dictionaries about removing records while sequencing them??? */
      db_rm(db, key); 
    status=db_seq(db, DICT_SEQ_FUN_NEXT, (void *)&key, (void *)&value);
  }
  db_close(db);
  return status;
}
