//#define USE_MAPS
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys_defs.h>
#include <mymalloc.h>
#include <vstring.h>
#include "auth.h"
#include "optm.h"

int main(int argc, char ** argv)
{
  char * username, * clientip, * dbname, * scheme, * orig;
  VSTRING * vstime;

#ifdef USE_OPTIMIZERS
  optm_t * optm;
  void * topass;
#endif 
  DBTYPE * db;
  optmdb_t * optdb;

  /* Get the needed configuration parameters and environment variables */
  username=getenv("AUTHENTICATED");
  clientip=getenv("TCPREMOTEIP");
  dbname=getenv("LOGINDB");

  /* If no db is provided, use the default name */
  if(dbname == NULL || *dbname == '\0')
    dbname=DEFAULTDB;
 
  orig=mystrdup(dbname);
  if(orig == NULL) {
    fprintf(stderr, "auth_clientdb: out of memory.\n");
    return 1;
  }
  
  /* If the user has been authenticated, add him to the database */
  if(username != NULL && clientip != NULL && *username != '\0' && *clientip != '\0') {
    
    /* If the clientip is in IPV6, try to convert it in IPV4 (quite an heuristic function) */
    clientip=ip6to4(clientip);

    split_up(orig, &scheme, &dbname); 
    if(scheme == NULL) {
      fprintf(stderr, "auth_clientdb: wrong LOGINDB.\n");
      return 1;
    }

#ifdef USE_OPTIMIZERS
    /* Look up if we have an optimized procedure for the indicated scheme */
    optm=optm_lookup(scheme);
    if(optm != NULL && optm->add != NULL) {
      optdb=optm->open(dbname);
      optm->add(optdb, clientip, (int)(time(NULL)));
      optm->close(optdb);
    }
    else {
#endif
      /* Convert time to an ascii number */
      vstime=vstring_alloc(11);
      vstring_sprintf(vstime, "%d", (int)(time(NULL)));

      db=db_open3( scheme, dbname);
      db_add(db, clientip, vstring_str(vstime));
      db_close(db);

      vstring_free(vstime);
#ifdef USE_OPTIMIZERS
    }
#endif
  }

  myfree(orig);

  /* Anyway, exec next module in chain */
  if( argv[1] == NULL || argc < 2 )
    return 1;

  execv(argv[1], &argv[1]);
  perror(argv[1]);

  return 1;
}
